@@ -18,7 +18,7 @@
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
  *   51 Franklin Street, Fifth Floor                                       *
- *   Boston, MA  02111-1307, USA.                                          *
+ *   Boston, MA 02110-1301, USA.                                          *
  ***************************************************************************/
 
 //Own
@@ -61,6 +61,7 @@
 #include "core/fileio.h"
 #include "core/path.h"
 #include "core/root.h"
+#include "core/splashdelegate.h"
 #include "core/splashpreviewjob.h"
 #include "settings.h"
 
@@ -70,7 +71,7 @@ K_EXPORT_PLUGIN( GRUBFactory( "kgrubeditor" ) )
 KGRUBEditor::KGRUBEditor( QWidget *parent, const QVariantList &list ) : KCModule( GRUBFactory::componentData(), parent )
 {
 	Q_UNUSED( list )
-	KAboutData *about = new KAboutData( "kgrubeditor", 0, ki18n( "KGRUBEditor" ), "0.8.1", ki18n( "A KDE utility, that edits GRUB's configuration files through an inituitive user interface." ), KAboutData::License_GPL, ki18n( "Copyright (C) 2008 Konstantinos Smanis" ), KLocalizedString(), "http://sourceforge.net/projects/kgrubeditor", "kon.smanis@gmail.com" );
+	KAboutData *about = new KAboutData( "kgrubeditor", 0, ki18n( "KGRUBEditor" ), "0.8.5b", ki18n( "A KDE utility, that edits GRUB's configuration files through an inituitive user interface." ), KAboutData::License_GPL, ki18n( "Copyright (C) 2008 Konstantinos Smanis" ), KLocalizedString(), "http://sourceforge.net/projects/kgrubeditor", "kon.smanis@gmail.com" );
 	about->addAuthor( ki18n( "Κonstantinos Smanis" ), ki18n( "Developer" ), "kon.smanis@gmail.com" );
 	about->addCredit( ki18n( "Andreas Theodosiou" ), ki18n( "Application's Icon Designer" ), "andreasabu@gmail.com" );
 	about->addCredit( ki18n( "Dimitris Palyvos-Giannas" ), ki18n( "Various help with the application" ), "jimaras@gmail.com" );
@@ -84,7 +85,6 @@ KGRUBEditor::KGRUBEditor( QWidget *parent, const QVariantList &list ) : KCModule
 	setupConnections();
 
 	enableEntryActions( false );
-	m_loading = true;
 }
 
 //SETUP FUNCTIONS
@@ -100,6 +100,10 @@ void KGRUBEditor::setupObjects()
 	m_gfxmenuDirWatchDeletion = new KDirWatch( this );
 	if ( !GrubSettings.gfxMenu().isEmpty() )
 		m_gfxmenuDirWatchDeletion->addFile( Core::Path::convertToGenericPath( GrubSettings.gfxMenu() ) );
+
+	m_splashModel = new SplashModel( this );
+	ui_background.kcombobox_splashlist->setModel( m_splashModel );
+	ui_background.kcombobox_splashlist->setItemDelegate( new SplashDelegate( this ) );
 }
 void KGRUBEditor::setupConnections()
 {
@@ -130,8 +134,8 @@ void KGRUBEditor::setupConnections()
 	connect( ui_password.kpushbutton_delete, SIGNAL( pressed() ), SLOT( deletePassword() ) );
 	//background
 	connect( ui_background.kcombobox_splashlist, SIGNAL( activated( int ) ), SLOT( updateSplashImage( int ) ) );
-	connect( ui_background.gruburlrequester_splash, SIGNAL( pathChanged( const QString & ) ), SLOT( splashImagePathChanged( const QString & ) ) );
-	connect( ui_background.gruburlrequester_splash, SIGNAL( pathCleared() ), SLOT( clearSplashImage() ) );
+	connect( ui_background.kcombobox_splashlist->model(), SIGNAL( modelReset() ), SLOT( showSplashImage() ) );
+	connect( ui_background.toolButton_browse, SIGNAL( clicked() ), SLOT( browseSplashImage() ) );
 	connect( ui_background.kpushbutton_create, SIGNAL( pressed() ), SLOT( createSplashImage() ) );
 	connect( ui_background.kpushbutton_getSplash, SIGNAL( pressed() ), SLOT( getNewSplashImages() ) );
 	connect( ui_background.kpushbutton_preview, SIGNAL( pressed() ), SLOT( previewSplashImage() ) );
@@ -238,30 +242,9 @@ void KGRUBEditor::showPassword()
 void KGRUBEditor::showSplashImage()
 {
 	if ( !GrubSettings.splashImage().isEmpty() )
-	{
-		const QString splashImage = Core::Path::convertToGenericPath( GrubSettings.splashImage() );
-		if ( QFile::exists( splashImage ) )
-		{
-			for ( int i = 1; i < ui_background.kcombobox_splashlist->count(); i++ )
-			{
-				if ( ui_background.kcombobox_splashlist->itemData( i ).toString() == splashImage )
-				{
-					ui_background.kcombobox_splashlist->setCurrentIndex( i );
-					break;
-				}
-			}
-		}
-		else
-		{
-			KMessageBox::sorry( this, i18nc( "@info", "The specified splash image <filename>%1</filename> was not found. No splash image will be used.", splashImage ) );
-			clearSplashImage();
-			return;
-		}
-	}
+		ui_background.kcombobox_splashlist->setCurrentIndex( m_splashModel->indexOf( Core::Path::convertToGenericPath( GrubSettings.splashImage() ) ) );
 	else
-		ui_background.kcombobox_splashlist->setCurrentIndex( 0 );
-
-	ui_background.gruburlrequester_splash->setPath( GrubSettings.splashImage() );
+		ui_background.kcombobox_splashlist->setCurrentIndex( ui_background.kcombobox_splashlist->count() - 1 );
 }
 void KGRUBEditor::showGfxMenu()
 {
@@ -321,7 +304,6 @@ void KGRUBEditor::showInput( const int entryRow, const int mapListRow, QWidget*
 	showHiddenMenu();
 	showTimeout();
 	showPassword();
-	showSplashImage();
 	showGfxMenu();
 	showColors();
 	showMaps( mapListRow );
@@ -385,7 +367,7 @@ void KGRUBEditor::appendSplashDirectory( const QString &directory )
 
 		monitorSplashDirectory( fi.absoluteFilePath() );
 
-		createSplashPreviews();
+		m_splashModel->reload();
 	}
 }
 //SLOTS
@@ -403,33 +385,6 @@ void KGRUBEditor::defaultUpdated()
 	}
 }
 
-void KGRUBEditor::createSplashPreviews()
-{
-	if ( !ThreadWeaver::Weaver::instance()->isIdle() )
-		return;
-
-	kDebug() << "Creating splash previews";
-	ui_background.kcombobox_splashlist->clear();
-	QPixmap pixmap( 64, 48 );
-	pixmap.fill( Qt::black );
-	ui_background.kcombobox_splashlist->addItem( QIcon( pixmap ), i18n( "No Splash Image" ) );
-
-	ThreadWeaver::JobCollection *jobCollection = new ThreadWeaver::JobCollection( this );
-	foreach( const QString &path, Settings::splashDirs() )
-	{
-		Core::Root::makeReadableByUserGroup( path, "*.xpm.gz", this );
-
-		foreach( const QString &splashImage, QDir( path ).entryList( QStringList() << "*.xpm.gz", QDir::Files ) )
-		{
-			SplashPreviewJob *splashPreviewJob = new SplashPreviewJob( path + "/" + splashImage, this );
-			connect( splashPreviewJob, SIGNAL( done( ThreadWeaver::Job * ) ), SLOT( appendSplashImagePreview( ThreadWeaver::Job * ) ) );
-			jobCollection->addJob( splashPreviewJob );
-		}
-	}
-	connect( jobCollection, SIGNAL( done( ThreadWeaver::Job * ) ), SLOT( splashImageThumbPreviewsDone( ThreadWeaver::Job * ) ) );
-	ThreadWeaver::Weaver::instance()->enqueue( jobCollection );
-	kDebug() << "Finished creating splash previews";
-}
 void KGRUBEditor::splashImageCreated( const QString &path )
 {
 	const QFileInfo fi( path );
@@ -440,7 +395,7 @@ void KGRUBEditor::splashImageCreated( const QString &path )
 	m_splashDirWatchDeletion->addFile( path );
 
 	SplashPreviewJob *splashPreviewJob = new SplashPreviewJob( path, this );
-	connect( splashPreviewJob, SIGNAL( done( ThreadWeaver::Job * ) ), SLOT( appendSplashImagePreview( ThreadWeaver::Job * ) ) );
+	connect( splashPreviewJob, SIGNAL( done( ThreadWeaver::Job * ) ), m_splashModel, SLOT( appendSplashImagePreview( ThreadWeaver::Job * ) ) );
 	ThreadWeaver::Weaver::instance()->enqueue( splashPreviewJob );
 }
 void KGRUBEditor::splashImageDeleted( const QString &path )
@@ -448,29 +403,10 @@ void KGRUBEditor::splashImageDeleted( const QString &path )
 	kDebug() << "Splash Image" << path << "was deleted";
 	m_splashDirWatchDeletion->removeFile( path );
 
-	for ( int i = 1; i < ui_background.kcombobox_splashlist->count(); i++ )
-	{
-		if ( ui_background.kcombobox_splashlist->itemData( i ).toString() == path )
-		{
-			if ( ui_background.kcombobox_splashlist->currentIndex() == i )
-				clearSplashImage();
-			ui_background.kcombobox_splashlist->removeItem( i );
-			return;
-		}
-	}
-}
-void KGRUBEditor::appendSplashImagePreview( ThreadWeaver::Job *job )
-{
-	SplashPreviewJob *splashJob = dynamic_cast< SplashPreviewJob * >( job );
-	if ( splashJob && ui_background.kcombobox_splashlist->findData( splashJob->absoluteFilePath() ) == -1 )
-	{
-		ui_background.kcombobox_splashlist->addItem( QIcon( QPixmap::fromImage( splashJob->splashImageThumb() ) ), QString( QFileInfo( splashJob->absoluteFilePath() ).fileName() ).remove( ".xpm.gz" ), QString( splashJob->absoluteFilePath() ) );
-	}
-}
-void KGRUBEditor::splashImageThumbPreviewsDone( ThreadWeaver::Job *job )
-{
-	Q_UNUSED( job )
-	showSplashImage();
+	if ( path == ui_background.kcombobox_splashlist->itemText( ui_background.kcombobox_splashlist->currentIndex() ) )
+		clearSplashImage();
+
+	m_splashModel->removeSplashImagePreview( path );
 }
 void KGRUBEditor::splashImagePreviewDone( ThreadWeaver::Job *job )
 {
@@ -729,24 +665,14 @@ void KGRUBEditor::deletePassword()
 
 void KGRUBEditor::updateSplashImage( const int previewRow )
 {
-	if ( previewRow == 0 )
+	if ( previewRow == ui_background.kcombobox_splashlist->count() - 1 )
 	{
 		clearSplashImage();
 		return;
 	}
 
-	QString grubPath = Core::Path::convertToGRUBPath( ui_background.kcombobox_splashlist->itemData( previewRow ).toString() );
-	updateSplashImage( grubPath );
-}
-void KGRUBEditor::splashImagePathChanged( const QString &path )
-{
-	appendSplashDirectory( QFileInfo( path ).dir().path() );
-	updateSplashImage( path );
-}
-void KGRUBEditor::updateSplashImage( const QString &path )
-{
-	GrubSettings.setSplashImage( path );
-	showSplashImage();
+	QString grubPath = Core::Path::convertToGRUBPath( ui_background.kcombobox_splashlist->itemText( previewRow ) );
+	GrubSettings.setSplashImage( grubPath );
 	emit changed( true );
 }
 void KGRUBEditor::clearSplashImage()
@@ -758,6 +684,25 @@ void KGRUBEditor::clearSplashImage()
 		emit changed( true );
 	}
 }
+void KGRUBEditor::browseSplashImage()
+{
+	KFileDialog dlgOpen( KUrl(), "*.xpm.gz|" + i18n( "GRUB Splash Image (*.xpm.gz)" ), this );
+	dlgOpen.setCaption( i18nc( "@window:title", "Select GRUB Splash Image" ) );
+	dlgOpen.setOperationMode( KFileDialog::Opening );
+	if ( !dlgOpen.exec() )
+		return;
+
+	KUrl imageFilename = dlgOpen.selectedUrl();
+	QString parentDir = QFileInfo( imageFilename.path() ).dir().path();
+
+	GrubSettings.setSplashImage( Core::Path::convertToGRUBPath( imageFilename.path() ) );
+	emit changed( true );
+
+	if ( Settings::splashDirs().contains( parentDir ) )
+		showSplashImage();
+	else
+		appendSplashDirectory( QFileInfo( imageFilename.path() ).dir().path() );
+}
 void KGRUBEditor::createSplashImage()
 {
 	KFileDialog dlgOpen( KUrl(), "*.bmp *.jpe *.jpg *.jpeg *.png *.ppm *.pgm *.pnm *.tiff *.tga *.xbm *.xpm|" + i18n( "Image Files" ), this );
@@ -783,11 +728,16 @@ void KGRUBEditor::createSplashImage()
 }
 void KGRUBEditor::getNewSplashImages()
 {
-	Core::Root::makeWritableByUserGroup( "/boot/grub/splashimages", this );
+	const QString downloadPath( "/boot/grub/splashimages" );
+	Core::Root::makeDirWritableByUserGroup( downloadPath, this );
 
-	KNS::Engine engine( 0 );
+	//If directory doesn't exist we should start monitoring it.
+	if ( !m_splashDirWatchCreation->contains( downloadPath ) )
+		m_splashDirWatchCreation->addDir( downloadPath, KDirWatch::WatchFiles );
+
+	KNS::Engine engine( this );
 	if ( engine.init( "kgrubeditor_splash.knsrc" ) )
-		KNS::Entry::List entries = engine.downloadDialogModal( this );
+		engine.downloadDialogModal( this );
 }
 void KGRUBEditor::previewSplashImage()
 {
@@ -837,8 +787,8 @@ void KGRUBEditor::clearGfxmenu()
 void KGRUBEditor::addMap()
 {
 	GRUB::ComplexCommand::Map tmp_map;
-	MapAssistant *mapAssistant = new MapAssistant( &tmp_map, this );
-	if ( mapAssistant->exec() )
+	MapEditor *mapEditor = new MapEditor( &tmp_map, this );
+	if ( mapEditor->exec() )
 	{
 		GrubSettings.addMap( tmp_map );
 		showMaps( ui_map.klistwidget_maps->count() );
@@ -851,8 +801,8 @@ void KGRUBEditor::editMap()
 		return;
 
 	GRUB::ComplexCommand::Map tmp_map = GrubSettings.maps().at( ui_map.klistwidget_maps->currentRow() );
-	MapAssistant *mapAssistant = new MapAssistant( &tmp_map, this );
-	if ( mapAssistant->exec() )
+	MapEditor *mapEditor = new MapEditor( &tmp_map, this );
+	if ( mapEditor->exec() )
 	{
 		GrubSettings.replaceMap( ui_map.klistwidget_maps->currentRow(), tmp_map );
 		showMaps( ui_map.klistwidget_maps->currentRow() );
@@ -864,7 +814,7 @@ void KGRUBEditor::removeMap()
 	if ( !ui_map.klistwidget_maps->currentItem() )
 		return;
 
-	if ( KMessageBox::questionYesNo( this, i18nc( "@info", "Are you sure you want to remove this map?" ), i18nc( "@window:title", "Confirmation" ) ) )
+	if ( KMessageBox::questionYesNo( this, i18nc( "@info", "Are you sure you want to remove this map?" ) ) )
 	{
 		GrubSettings.removeMap( ui_map.klistwidget_maps->currentRow() );
 		showMaps( ui_map.klistwidget_maps->currentRow() - 1 );
@@ -906,6 +856,7 @@ void KGRUBEditor::preferences()
 	dialog->addPage( pathsSettings, i18nc( "@item:inlist", "Paths" ) )->setIcon( KIcon( "folder-txt" ) );
 	connect( dialog, SIGNAL( settingsChanged( QString ) ), SLOT( refresh() ) );
 	dialog->setAttribute( Qt::WA_DeleteOnClose );
+	dialog->setFaceType( KPageDialog::Plain );
 	dialog->show();
 }
 
@@ -924,11 +875,7 @@ void KGRUBEditor::refresh( int entryRow, int mapListRow, QWidget* focusWidget )
 	if ( !KIO::NetAccess::exists( KUrl( Settings::menulst().path() + "_original" ), true, this ) )
 		Core::Backup::backup( Settings::menulst(), KUrl( Settings::menulst().path() + "_original" ), this, false );
 
-	if ( m_loading )
-	{
-		m_loading = false;
-		createSplashPreviews();
-	}
+	m_splashModel->reload();
 }
 void KGRUBEditor::load()
 {
